@@ -9,6 +9,7 @@ Example:
     await inst.start()
     # TCP server is running now.
 """
+
 from __future__ import annotations
 
 import asyncio
@@ -84,8 +85,7 @@ class ServerManager(typing.Protocol):
     @contextmanager
     def register_connection(
         self, connection_id: tuple | str, handler: ProxyConnectionHandler
-    ):
-        ...  # pragma: no cover
+    ): ...  # pragma: no cover
 
 
 class ServerInstance(Generic[M], metaclass=ABCMeta):
@@ -204,7 +204,9 @@ class ServerInstance(Generic[M], metaclass=ABCMeta):
             else:
                 handler.layer.context.client.sockname = original_dst
                 handler.layer.context.server.address = original_dst
-        elif isinstance(self.mode, (mode_specs.WireGuardMode, mode_specs.LocalMode)):
+        elif isinstance(
+            self.mode, (mode_specs.WireGuardMode, mode_specs.LocalMode)
+        ):  # pragma: no cover on platforms without wg-test-client
             handler.layer.context.server.address = writer.get_extra_info(
                 "remote_endpoint", handler.layer.context.client.sockname
             )
@@ -325,7 +327,9 @@ class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
     server_key: str
     client_key: str
 
-    def make_top_layer(self, context: Context) -> Layer:
+    def make_top_layer(
+        self, context: Context
+    ) -> Layer:  # pragma: no cover on platforms without wg-test-client
         return layers.modes.TransparentProxy(context)
 
     @property
@@ -387,7 +391,11 @@ class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
     def client_conf(self) -> str | None:
         if not self._server:
             return None
-        host = local_ip.get_local_ip() or local_ip.get_local_ip6()
+        host = (
+            self.mode.listen_host(ctx.options.listen_host)
+            or local_ip.get_local_ip()
+            or local_ip.get_local_ip6()
+        )
         port = self.mode.listen_port(ctx.options.listen_port)
         return textwrap.dedent(
             f"""
@@ -414,7 +422,9 @@ class WireGuardServerInstance(ServerInstance[mode_specs.WireGuardMode]):
         finally:
             self._server = None
 
-    async def wg_handle_stream(self, stream: mitmproxy_rs.Stream) -> None:
+    async def wg_handle_stream(
+        self, stream: mitmproxy_rs.Stream
+    ) -> None:  # pragma: no cover on platforms without wg-test-client
         await self.handle_stream(stream, stream)
 
 
